package examples.sparksql;

import com.aliyun.odps.data.Record;
import com.tzld.piaoquan.ad.engine.commons.base.AdItemFeature;
import com.tzld.piaoquan.ad.engine.commons.base.UserAdFeature;
import com.tzld.piaoquan.data.base.UserFeature;
import examples.dataloader.AdRedisFeatureConstructor;
import examples.dataloader.RecommRedisFeatureConstructor;
import org.apache.spark.SparkConf;
import org.apache.spark.aliyun.odps.OdpsOps;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

import java.util.HashMap;
import java.util.Map;


public class SparkAdFeaToRedisLoader {

    private static final String userKeyFormat = "user:ad:%s";

    private static final String adKeyFormat = "ad:%s";


    public static RedisTemplate<String, String> buildRedisTemplate() {
        RedisStandaloneConfiguration rsc = new RedisStandaloneConfiguration();
        rsc.setPort(6379);
        rsc.setPassword("Wqsd@2019");
        rsc.setHostName("r-bp1pi8wyv6lzvgjy5z.redis.rds.aliyuncs.com");
        RedisTemplate<String, String> template = new RedisTemplate<>();
        JedisConnectionFactory fac = new JedisConnectionFactory(rsc);
        fac.afterPropertiesSet();
        template.setDefaultSerializer(new StringRedisSerializer());
        template.setConnectionFactory(fac);
        template.afterPropertiesSet();
        return template;
    }

    public static void loadUserAdFeatureToRedis(RedisTemplate<String, String> redisTemplate, Record line) {
        Map<String, String> userFeaRedisFormat = new HashMap<String, String>();
        UserAdFeature userFeature = AdRedisFeatureConstructor.constructUserFeature(line);
        String key = String.format(userKeyFormat, userFeature.getKey());
        String value = userFeature.getValue();
        userFeaRedisFormat.put(key, value);
        redisTemplate.opsForValue().multiSet(userFeaRedisFormat);
    }


    public static void loadAdFeatureToRedis(RedisTemplate<String, String> redisTemplate, Record line) {
        Map<String, String> userFeaRedisFormat = new HashMap<String, String>();
        AdItemFeature adItemFeature = AdRedisFeatureConstructor.constructItemFeature(line);
        String key = String.format(userKeyFormat, adItemFeature.getKey());
        String value = adItemFeature.getValue();
        userFeaRedisFormat.put(key, value);
        redisTemplate.opsForValue().multiSet(userFeaRedisFormat);
    }


    public static void main(String[] args) {

        String partition = args[0];
        String accessId = "LTAIWYUujJAm7CbH";
        String accessKey = "RfSjdiWwED1sGFlsjXv0DlfTnZTG1P";
        String odpsUrl = "http://service.odps.aliyun.com/api";
        String tunnelUrl = "http://dt.cn-hangzhou.maxcompute.aliyun-inc.com";
        String project = "loghubods";
        String tableAdInfo = "alg_ad_item_info";
        String tableUserInfo = "alg_ad_user_info";


        SparkConf sparkConf = new SparkConf().setAppName("E-MapReduce Demo 3-2: Spark MaxCompute Demo (Java)");
        JavaSparkContext jsc = new JavaSparkContext(sparkConf);
        OdpsOps odpsOps = new OdpsOps(jsc.sc(), accessId, accessKey, odpsUrl, tunnelUrl);
        System.out.println("Read odps table...");


        // load Ad features
        JavaRDD<Record> readAdData = odpsOps.readTableWithJava(project, tableAdInfo, partition, null, Integer.valueOf(10));
        readAdData.foreachPartition(
                rowIterator -> {
                    RedisTemplate<String, String> redisTemplate = buildRedisTemplate();
                    rowIterator.forEachRemaining(line -> loadAdFeatureToRedis(redisTemplate, line));
                }
        );


        // load user features
        JavaRDD<Record> readUserData = odpsOps.readTableWithJava(project, tableUserInfo, partition, null, Integer.valueOf(10));
        readUserData.foreachPartition(
                rowIterator -> {
                    RedisTemplate<String, String> redisTemplate = buildRedisTemplate();
                    rowIterator.forEachRemaining(line -> loadUserAdFeatureToRedis(redisTemplate, line));
                }
        );


    }

}